package com.yonge.cooleshow.biz.dal.service.impl;

import cn.hutool.core.util.NumberUtil;
import com.alibaba.fastjson.JSON;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.yonge.cooleshow.auth.api.client.SysUserFeignService;
import com.yonge.cooleshow.biz.dal.entity.MusicFavorite;
import com.yonge.cooleshow.biz.dal.entity.MusicSheet;
import com.yonge.cooleshow.biz.dal.entity.MusicTag;
import com.yonge.cooleshow.biz.dal.entity.Subject;
import com.yonge.cooleshow.biz.dal.entity.SysUser;
import com.yonge.cooleshow.biz.dal.entity.TenantAlbumCategory;
import com.yonge.cooleshow.biz.dal.entity.TenantAlbumCategoryDetail;
import com.yonge.cooleshow.biz.dal.entity.TenantAlbumMusic;
import com.yonge.cooleshow.biz.dal.enums.ClientEnum;
import com.yonge.cooleshow.biz.dal.enums.SubjectTypeEnum;
import com.yonge.cooleshow.biz.dal.mapper.SysUserMapper;
import com.yonge.cooleshow.biz.dal.mapper.TenantAlbumMusicMapper;
import com.yonge.cooleshow.biz.dal.service.MusicFavoriteService;
import com.yonge.cooleshow.biz.dal.service.MusicSheetService;
import com.yonge.cooleshow.biz.dal.service.MusicTagService;
import com.yonge.cooleshow.biz.dal.service.SubjectService;
import com.yonge.cooleshow.biz.dal.service.TenantAlbumCategoryDetailService;
import com.yonge.cooleshow.biz.dal.service.TenantAlbumCategoryService;
import com.yonge.cooleshow.biz.dal.service.TenantAlbumMusicService;
import com.yonge.cooleshow.biz.dal.service.UserTenantAlbumRecordService;
import com.yonge.cooleshow.biz.dal.wrapper.TenantAlbumMusicWrapper;
import com.yonge.toolset.base.exception.BizException;
import jodd.util.StringUtil;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collection;
import java.util.Collections;
import java.util.List;
import java.util.stream.Collectors;
import java.util.stream.Stream;

/**
 * 机构专辑曲目
 * 2023-07-21 17:32:49
 */
@Slf4j
@Service
public class TenantAlbumMusicServiceImpl extends ServiceImpl<TenantAlbumMusicMapper, TenantAlbumMusic> implements TenantAlbumMusicService {


    @Autowired
    private UserTenantAlbumRecordService userTenantAlbumRecordService;

    @Autowired
    private MusicSheetService musicSheetService;

    @Autowired
    private SubjectService subjectService;

    @Autowired
    private MusicTagService musicTagService;

    @Autowired
    private TenantAlbumMusicMapper tenantAlbumMusicMapper;

    @Autowired
    private SysUserMapper sysUserMapper;


    @Autowired
    private TenantAlbumMusicService tenantAlbumMusicService;


    @Autowired
    private MusicFavoriteService musicFavoriteService;

    @Autowired
    private  SysUserFeignService sysUserFeignService;

    @Autowired
    private TenantAlbumCategoryDetailService tenantAlbumCategoryDetailService;

    /**
     * 查询详情
     * @param id 详情ID
     * @return TenantAlbumMusic
     */
    @Override
    public TenantAlbumMusic detail(Long id) {
        
        return baseMapper.selectById(id);
    }
    
    /**
     * 分页查询
     * @param page IPage<TenantAlbumMusic>
     * @param query TenantAlbumMusicWrapper.TenantAlbumMusicQuery
     * @return IPage<TenantAlbumMusic>
     */
    @Override
    public IPage<TenantAlbumMusicWrapper.StudentTenantAlbumMusic> selectPage(IPage<TenantAlbumMusicWrapper.StudentTenantAlbumMusic> page, TenantAlbumMusicWrapper.StudentTenantAlbumMusicQuery query) {
        com.yonge.cooleshow.auth.api.entity.SysUser user = sysUserFeignService.queryUserInfo();
        Long sysUserId = user.getId();

        String albumId = query.getAlbumId();
        if (StringUtil.isEmpty(albumId)){
            Long tenantId = query.getTenantId();
            //查询机构专辑Id
            List<TenantAlbumMusic> list = tenantAlbumMusicService.lambdaQuery().eq(TenantAlbumMusic::getTenantId, tenantId).list();
            if (CollectionUtils.isNotEmpty(list)){
                TenantAlbumMusic albumMusic = list.get(0);
                 albumId = albumMusic.getTenantAlbumId().toString();
            } else {
                throw new BizException("该机构未绑定相关专辑");
            }
        }
        SubjectTypeEnum subjectType = query.getSubjectType();
        String s = subjectType.toString();
        String type = query.getType();
        String level = query.getLevel();

        List<Long> musicSheetIds = tenantAlbumMusicMapper.selectMusicSheetIds(albumId, s, type, level);

        if (CollectionUtils.isEmpty(musicSheetIds)) {
            page.setTotal(0);
            return page.setRecords(new ArrayList<>());
        }
        TenantAlbumMusicWrapper.StudentMusicSheetQuery musicSheetQuery = new TenantAlbumMusicWrapper.StudentMusicSheetQuery();
        musicSheetQuery.setMusicSheetIds(musicSheetIds);
        musicSheetQuery.setMusicTagId(query.getMusicTagId());
        musicSheetQuery.setSubjectId(query.getSubjectId());
        musicSheetQuery.setKeyword(query.getKeyword());
        //把集合转为字符串
        String collect1 = musicSheetIds.stream()
                .map(String::valueOf)
                .collect(Collectors.joining(", "));
        musicSheetQuery.setCollect(collect1);

        List<TenantAlbumMusicWrapper.StudentTenantAlbumMusic> musicSheets = tenantAlbumMusicMapper.selectPage(page, musicSheetQuery);

        musicSheets.stream().forEach(m->{
            Long userId = m.getUserId();
            String musicSubject = m.getMusicSubject();

            //获取用户名
            if (userId != null){
                SysUser sysUser = sysUserMapper.selectById(userId);
                m.setUserName(sysUser.getUsername());
            }

            //获取声部名
            if (StringUtils.isNotEmpty(musicSubject)){
                List<Subject> subject = subjectService.findBySubjectByIdList(musicSubject);
                if (CollectionUtils.isNotEmpty(subject)){
                    Subject subject1 = subject.get(0);
                    m.setMusicSubjectName(subject1.getName());
                }
            }

            //是否收藏
            Long id = m.getId();
            List<MusicFavorite> list = musicFavoriteService.lambdaQuery()
                    .eq(MusicFavorite::getMusicSheetId, id)
                    .eq(MusicFavorite::getUserId,sysUserId)
                    .eq(MusicFavorite::getClientType,query.getClientType().equals(ClientEnum.TENANT_STUDENT)?ClientEnum.STUDENT:query.getClientType()).list();
            if (CollectionUtils.isNotEmpty(list)){
                m.setFavorite(true);
            }

        });


        return page.setRecords(musicSheets);
    }

    /**
     * 添加
     * @param tenantAlbumMusic TenantAlbumMusicWrapper.TenantAlbumMusic
     * @return Boolean
     */
    @Override
    public Boolean add(TenantAlbumMusicWrapper.TenantAlbumMusic tenantAlbumMusic) {

        return this.save(JSON.parseObject(tenantAlbumMusic.jsonString(), TenantAlbumMusic.class));
    }

    /**
     * 更新
     * @param tenantAlbumMusic TenantAlbumMusicWrapper.TenantAlbumMusic
     * @return Boolean
     */
    @Override
    public Boolean update(TenantAlbumMusicWrapper.TenantAlbumMusic tenantAlbumMusic){

        return this.updateById(JSON.parseObject(tenantAlbumMusic.jsonString(), TenantAlbumMusic.class));       
    }


    /**
     * 获取机构专辑曲目查询条件
     */
    @Override
    public TenantAlbumMusicWrapper.TenantAlbumMusicSelectData getTenantAlbumMusicQuery(TenantAlbumMusicWrapper.TenantAlbumMusicSelect query) {

        TenantAlbumMusicWrapper.TenantAlbumMusicSelectData data = new TenantAlbumMusicWrapper.TenantAlbumMusicSelectData();

        // 获取学生的专辑
        List<Long> albumIds = userTenantAlbumRecordService.getUseAlbumIdsByUserId(query.getUserId(), ClientEnum.STUDENT);
        if (CollectionUtils.isEmpty(albumIds)) {
            return data;
        }

        // 专辑曲目关联数据
        List<TenantAlbumMusic> list = this.lambdaQuery()
                .in(TenantAlbumMusic::getTenantAlbumId, albumIds)
                .eq(query.getSubjectType() !=null,TenantAlbumMusic::getSubjectType, query.getSubjectType())
                .eq(TenantAlbumMusic::getDelFlag, false)
                .list();
        if (CollectionUtils.isEmpty(list)) {
            return data;
        }

        List<Long> musicSheetIds = list.stream().map(o -> o.getMusicSheetId()).distinct().collect(Collectors.toList());
        if (CollectionUtils.isEmpty(musicSheetIds)) {
            return data;
        }

        Collection<MusicSheet> musicSheets = musicSheetService.listByIds(musicSheetIds);
        if (CollectionUtils.isEmpty(musicSheets)) {
            return data;
        }

        // 类型
        List<Long> typeIds =list.stream().map(TenantAlbumMusic::getType).distinct().filter(StringUtils::isNotBlank)
                .filter(NumberUtil::isNumber).map(Long::valueOf).collect(Collectors.toList());
        if (!typeIds.isEmpty()) {
            List<TenantAlbumCategoryDetail> typeList = tenantAlbumCategoryDetailService.lambdaQuery()
                    .in(TenantAlbumCategoryDetail::getId, typeIds).list();
            List<String> typeNameList = typeList.stream()
                    .map(TenantAlbumCategoryDetail::getValue).collect(Collectors.toList());
            data.setTypes(typeNameList);
            data.setTypeList(typeList);
        } else {
            data.setTypes(new ArrayList<>());
        }
        // 级别
        List<Long> levelIds =list.stream().map(TenantAlbumMusic::getLevel).distinct().filter(StringUtils::isNotBlank)
                .filter(NumberUtil::isNumber).map(Long::valueOf).collect(Collectors.toList());
        if (!levelIds.isEmpty()) {
            List<TenantAlbumCategoryDetail> levelList = tenantAlbumCategoryDetailService.lambdaQuery()
                    .in(TenantAlbumCategoryDetail::getId, levelIds).list();
            List<String> levelNameList = levelList.stream()
                    .map(TenantAlbumCategoryDetail::getValue).collect(Collectors.toList());
            data.setLevels(levelNameList);
            data.setLevelList(levelList);
        } else {
            data.setLevels(new ArrayList<>());
        }

        // 声部
        List<Long> subjectIds = musicSheets.stream()
                .map(MusicSheet::getMusicSubject)
                .filter(StringUtils::isNotBlank).distinct()
                .map(Long::parseLong).collect(Collectors.toList());
        if (CollectionUtils.isNotEmpty(subjectIds)) {
            List<Subject> subjects = subjectService.findBySubjectByIdList(subjectIds);
            data.setSubjects(subjects);
        }

        // 标签
        List<Long> musicTagIds = musicSheets.stream().flatMap(o -> {
            if (StringUtils.isNotBlank(o.getMusicTag())) {
                return Arrays.stream(o.getMusicTag().split(","));
            }
            return Stream.empty();

        }).filter(StringUtils::isNotBlank).distinct().map(Long::parseLong).collect(Collectors.toList());
        if (!CollectionUtils.isEmpty(musicTagIds)) {

            List<MusicTag> tagList = musicTagService.listByIds(musicTagIds).parallelStream().collect(Collectors.toList());
            data.setTags(tagList);
        }

        return data;

    }

    @Override
    public List<Long> getMusicIdsByIds(List<Long> tenantAlbumIds) {

        if (CollectionUtils.isEmpty(tenantAlbumIds)) {
            return Collections.emptyList();
        }
        List<TenantAlbumMusic> list = this.lambdaQuery().in(TenantAlbumMusic::getTenantAlbumId, tenantAlbumIds).list();
        if (CollectionUtils.isEmpty(list)) {
            return Collections.emptyList();
        }
        return list.stream().map(TenantAlbumMusic::getMusicSheetId).collect(Collectors.toList());
    }


    /**
     * 获取机构专辑曲目ID集合
     *
     * @param tenantId 机构ID
     */
    @Override
    public List<Long> getMusicIdsByTenantIds(Long tenantId) {
        return baseMapper.selectMusicIdsByTenantIds(tenantId);
    }
}