package com.yonge.cooleshow.biz.dal.service.impl;

import com.alibaba.fastjson.JSON;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.yonge.cooleshow.biz.dal.entity.UserMusicStar;
import com.yonge.cooleshow.biz.dal.mapper.UserMusicMapper;
import com.yonge.cooleshow.biz.dal.mapper.UserMusicStarMapper;
import com.yonge.cooleshow.biz.dal.service.UserMusicStarService;
import com.yonge.cooleshow.biz.dal.wrapper.UserMusicStarWrapper;
import com.yonge.toolset.base.util.StringUtil;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.collections.CollectionUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * 用户曲目作品点赞记录
 * 2023-10-30 14:59:53
 */
@Slf4j
@Service
public class UserMusicStarServiceImpl extends ServiceImpl<UserMusicStarMapper, UserMusicStar> implements UserMusicStarService {

    @Autowired
    private UserMusicMapper userMusicMapper;

	/**
     * 查询详情
     * @param id 详情ID
     * @return UserMusicStar
     */
	@Override
    public UserMusicStar detail(Long id) {

        return baseMapper.selectById(id);
    }

    /**
     * 分页查询
     * @param page IPage<UserMusicStar>
     * @param query UserMusicStarWrapper.UserMusicStarQuery
     * @return IPage<UserMusicStar>
     */
    @Override
    public IPage<UserMusicStar> selectPage(IPage<UserMusicStar> page, UserMusicStarWrapper.UserMusicStarQuery query) {

        return page.setRecords(baseMapper.selectPage(page, query));
    }

    /**
     * 添加
     * @param userMusicStar UserMusicStarWrapper.UserMusicStar
     * @return Boolean
     */
    @Override
    public Boolean add(UserMusicStarWrapper.UserMusicStar userMusicStar) {

        return this.save(JSON.parseObject(userMusicStar.jsonString(), UserMusicStar.class));
    }

    /**
     * 更新
     * @param userMusicStar UserMusicStarWrapper.UserMusicStar
     * @return Boolean
     */
    @Override
    public Boolean update(UserMusicStarWrapper.UserMusicStar userMusicStar){

        return this.updateById(JSON.parseObject(userMusicStar.jsonString(), UserMusicStar.class));
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public void star(UserMusicStarWrapper.Star star) {

        this.lambdaUpdate()
            .eq(UserMusicStar::getUserId, star.getUserId())
            .eq(UserMusicStar::getUserMusicId, star.getUserMusicId())
            .eq(UserMusicStar::getClientType, star.getClientType())
            .remove();

        if (star.getStar()) {
            UserMusicStar userMusicStar = new UserMusicStar();
            userMusicStar.setClientType(star.getClientType());
            userMusicStar.setUserId(star.getUserId());
            userMusicStar.setUserMusicId(star.getUserMusicId());
            this.save(userMusicStar);
        }
        userMusicMapper.updateStar(star.getUserMusicId());


    }

    @Override
    public Map<Long, Integer> getStarCountMapByUserMusicIds(List<Long> userMusicIds) {
        if (CollectionUtils.isEmpty(userMusicIds)) {
            return new HashMap<>();
        }
        List<UserMusicStarWrapper.StarCount> starCounts = baseMapper.getStarCountMapByUserMusicIds(userMusicIds);
        Map<Long, Integer> starCountMap = new HashMap<>();
        for (UserMusicStarWrapper.StarCount starCount : starCounts) {
            starCountMap.put(starCount.getUserMusicId(), starCount.getStarCount());
        }
        return starCountMap;
    }

    @Override
    public Map<Long, Boolean> getStarFlagMapByUserMusicIds(List<Long> userMusicIds, String clientType, String userId) {
        if (CollectionUtils.isEmpty(userMusicIds)) {
            return new HashMap<>();
        }
        if (StringUtil.isEmpty(clientType)) {
            return new HashMap<>();
        }
        if (StringUtil.isEmpty(userId)) {
            return new HashMap<>();
        }
        List<UserMusicStar> list = this.lambdaQuery()
            .eq(UserMusicStar::getClientType, clientType)
            .eq(UserMusicStar::getUserId, userId)
            .in(UserMusicStar::getUserMusicId, userMusicIds)
            .list();
        if (CollectionUtils.isEmpty(list)) {
            return new HashMap<>();
        }
        return list.stream().collect(Collectors.toMap(UserMusicStar::getUserMusicId, o->true));
    }
}
